"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cli_1 = require("../cli");
const ssh_1 = require("../session/ssh");
const path = require("path");
const chalk_1 = require("chalk");
const snipsfile_1 = require("../models/snipsfile");
exports.command = 'deploy';
exports.desc = 'Deploy Snipsfile';
exports.builder = {
    snipsfile_path: {
        demandOption: false,
        describe: 'Location of the Snipsfile',
        type: 'string',
        alias: 'sp',
    },
    assistant_path: {
        demandOption: false,
        describe: `Location of the assistant, zip or folder. It's name should be also reflected in the Snipsfile`,
        type: 'string',
        alias: 'ap',
    },
    verbose: {
        demandOption: false,
        describe: 'Include remote logs',
        type: 'boolean',
        alias: 'v',
    },
};
exports.handler = (argv) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    let snipsfilePath = JSON.parse(JSON.stringify(argv)).snipsfile_path;
    const assistantPath = JSON.parse(JSON.stringify(argv)).assistant_path;
    const verbose = JSON.parse(JSON.stringify(argv)).verbose;
    if (snipsfilePath === undefined) {
        snipsfilePath = path.join(process.cwd(), 'Snipsfile');
    }
    const snipsfile = yield snipsfile_1.Snipsfile.parse(snipsfilePath)
        .catch(_ => {
        cli_1.cli.stream.done(false);
        cli_1.cli.stream.errorPlain(`Error: Could not locate Snipsfile`);
        process.exit();
    });
    if (snipsfile === undefined) {
        cli_1.cli.stream.done(false);
        cli_1.cli.stream.errorPlain(`Error: Could not locate Snipsfile`);
        return process.exit();
    }
    const ssh = new ssh_1.SSHService();
    yield ssh.connect()
        .catch(e => {
        cli_1.cli.stream.error(e);
        process.exit();
    });
    if (assistantPath !== undefined) {
        cli_1.cli.stream.loading(`Deploying assistant to ${ssh.credentials.hostname}...`);
        yield ssh.installAssistant(assistantPath)
            .catch(e => {
            cli_1.cli.stream.error('Unable to install the assistant:' + e.message);
            ssh.disconnect();
            process.exit();
        });
        cli_1.cli.stream.success();
    }
    cli_1.cli.stream.loading(`Copying Snipsfile to device...`);
    yield ssh.putFile(snipsfilePath, 'Snipsfile')
        .catch(e => {
        cli_1.cli.stream.done(false);
        cli_1.cli.stream.errorPlain(`Error deploying assistant: ${e.message}`);
        ssh.disconnect();
        process.exit();
    });
    cli_1.cli.stream.done();
    let consoleUsername = '';
    let consolePassword = '';
    if (snipsfile.assistantId !== undefined) {
        const responses = yield cli_1.cli.prompt.promptQuestions([
            {
                name: 'username',
                type: 'username',
                message: 'Enter username for the Snips Console:',
                validate: value => {
                    if (value.length) {
                        return true;
                    }
                    return 'Username cannot be blank';
                },
            },
            {
                name: 'password',
                type: 'password',
                message: 'Enter password for Snips Console:',
                validate: _ => {
                    return true;
                },
            },
        ]);
        consoleUsername = responses['username'];
        consolePassword = responses['password'];
    }
    cli_1.cli.stream.loading('Installing dependencies. This may take a few minutes...');
    yield ssh.runSnipsmanagerInstall(consoleUsername, consolePassword, output => {
        if (verbose) {
            cli_1.cli.stream.done();
            cli_1.cli.stream.println(output);
        }
    });
    cli_1.cli.stream.done();
    cli_1.cli.stream.loading('Starting assistant...');
    yield ssh.runSnipsManager(output => {
        cli_1.cli.stream.done(true);
        if (verbose) {
            cli_1.cli.stream.println(output);
        }
    });
    const assistant = yield ssh.getInstalledAssistant();
    cli_1.cli.stream.done();
    cli_1.cli.stream.println();
    if (assistant !== undefined) {
        cli_1.cli.stream.success(`Snips assistant is now running. Say ${chalk_1.default.blue(assistant.hotword)} to start!`);
    }
    else {
        cli_1.cli.stream.success(`Snips assistant is now running`);
    }
    ssh.disconnect();
    process.exit();
});
