"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fsExtra = require("fs-extra");
const nodeSsh = require("node-ssh");
const os = require("os");
const path = require("path");
const shelljs = require("shelljs");
const audiodevice_1 = require("../utils/audiodevice");
const assistant_1 = require("../models/assistant");
const config_1 = require("../utils/config");
const errors_1 = require("../utils/errors");
const promises_1 = require("../utils/promises");
const pgpKey_1 = require("../assets/pgpKey");
exports.snipsRemoteAuthorizedKeysFilename = '~/.ssh/authorized_keys';
const snipsRSAKeyName = 'id_rsa_snips';
const snipsRSAKeyFilePath = path.join(os.homedir(), '.ssh', snipsRSAKeyName);
exports.availableSnipsServices = ['snips-hotword', 'snips-queries', 'snips-analytics', 'snips-dialogue', 'snips-asr', 'snips-tts', 'snips-audio-server', 'snipsmanager'];
class SSHService {
    constructor() {
        this.ssh = new nodeSsh();
    }
    connect() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const credentials = yield config_1.Config.getCredentials()
                .catch(_ => { errors_1.newError(`No credentials found`, errors_1.NO_CREDENTIALS_FOUND); });
            if (credentials === undefined) {
                throw errors_1.newError(`No credentials found`, errors_1.NO_CREDENTIALS_FOUND);
            }
            return this.connectInternal(credentials)
                .catch(e => {
                this.disconnect();
                throw errors_1.newError(e.message, errors_1.SSH_CONNECT_ERROR);
            });
        });
    }
    retryConnect(tick, limit = 5) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let isConnected = false;
            let numberOfRetries = 0;
            while (isConnected === false && limit > numberOfRetries) {
                yield promises_1.timeout(5000);
                tick();
                yield this.connect()
                    .then(_ => isConnected = true)
                    .catch(_ => { });
                numberOfRetries += 1;
            }
            return isConnected;
        });
    }
    newConnection(credentials, password, onConnected, onSSHCopyId, onCredentialsSaved) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connectInternal(credentials, password)
                .then(_ => onConnected())
                .catch(e => {
                throw new Error(`Error connecting to ${credentials.hostname}: ${e.message}`);
            });
            yield this.sshCopyId()
                .then(_ => onSSHCopyId())
                .catch(e => {
                this.disconnect();
                throw new Error(`Failed to copy SSH public key: ${e.message}`);
            });
            yield config_1.Config.deleteCredentials().catch(_ => { });
            yield config_1.Config.saveCredentialsOnDisk(credentials)
                .then(_ => onCredentialsSaved())
                .catch(e => {
                this.disconnect();
                throw new Error(`Failed to save credentials: ${e.message}`);
            });
        });
    }
    connectInternal(credentials, password) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.credentials = credentials;
            if (password != null) {
                return this.ssh.connect({
                    password,
                    host: this.credentials.hostname,
                    username: this.credentials.username,
                });
            }
            return this.ssh.connect({
                host: this.credentials.hostname,
                username: this.credentials.username,
                privateKey: snipsRSAKeyFilePath,
            });
        });
    }
    disconnect() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.ssh.dispose();
        });
    }
    reboot() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.ssh.exec('sudo reboot');
        });
    }
    isAssistantInstalled() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`if [ -d /usr/share/snips/assistant ]; then echo 1; else echo 0; fi`)
                .then(result => { return result === 1; });
        });
    }
    getInstalledAssistant() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const assistantFilename = '/usr/share/snips/assistant/assistant.json';
            return this.ssh.exec(`if [ -f ${assistantFilename} ]; then cat ${assistantFilename}; else echo ""; fi`)
                .then(result => {
                if (result === '') {
                    return undefined;
                }
                return assistant_1.Assistant.fromString(result);
            });
        });
    }
    getOSVersion() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec('cat /etc/os-release')
                .then(success => {
                const osVersion = success.split('\n')[0].replace('PRETTY_NAME="', '').replace('"', '');
                if (osVersion === null || osVersion.length === 0) {
                    return 'Unknown';
                }
                return osVersion;
            })
                .catch(_ => {
                return 'Unknown';
            });
        });
    }
    installSnipsPlatformDemo(listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog(`sudo apt-get install -y snips-platform-demo`, listener);
        });
    }
    aptGet(packageName, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog(`sudo apt-get install -y ${packageName}`, listener);
        });
    }
    isNetcatInstalled() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`if [ -x "$(command -v nc)" ]; then echo 1; else echo 0; fi`)
                .then(result => {
                if (!result) {
                    return this.dpkgCheckIfPackageIsInstalled('netcat');
                }
                return 1;
            })
                .then(result => result === 1 ? true : false);
        });
    }
    speakerTest() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo speaker-test -c2 -l2 -twav`);
        });
    }
    stopSpeakerTest() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`pkill speaker-test`);
        });
    }
    setHostname(newHostname) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const oldHostname = yield this.ssh.exec('cat /etc/hostname')
                .catch(_ => {
                throw new Error('You device hostname was not readable');
            });
            return this.ssh.exec(`sudo sed -i "s/${oldHostname}/${newHostname}/g" /etc/hosts; sudo sed -i "s/${oldHostname}/${newHostname}/g" /etc/hostname; sudo reboot`)
                .then((_) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.credentials.hostname = newHostname + '.local';
                return config_1.Config.saveCredentialsOnDisk(this.credentials);
            }))
                .catch((error) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (error.message.includes('sudo: unable to resolve host')) {
                    this.credentials.hostname = newHostname + '.local';
                    return config_1.Config.saveCredentialsOnDisk(this.credentials);
                }
            }));
        });
    }
    static fetchBootHostname(ips) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield Promise.all(ips.map((ip) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const client = new nodeSsh();
                const hostnames = yield client.connect({
                    host: ip,
                    username: 'pi',
                    password: 'raspberry',
                })
                    .then((_) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    let snipsHostname = yield client.exec('cat /boot/alias').catch(_ => '');
                    const hostname = yield client.exec('cat /etc/hostname').catch(_ => '');
                    if (snipsHostname.includes('No such file or directory')) {
                        snipsHostname = '';
                    }
                    return { hostname, snipsHostname };
                }))
                    .catch(_ => {
                    return { snipsHostname: '', hostname: '' };
                });
                return { ip, hostname: hostnames.hostname, snipsHostname: hostnames.snipsHostname };
            })));
        });
    }
    getCaptureLevel(card = 1) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`amixer -c ${card} cget name='Mic Capture Volume'`);
        });
    }
    setCaptureLevel(card = 1, level = 80) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`amixer -c ${card} cset name='Mic Capture Volume' ${level}%`);
        });
    }
    getVolumeLevel(card = 0) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`amixer -c ${card} cget name='PCM Playback Volume'`);
        });
    }
    setVolumeLevel(card = 0, level = 90) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`amixer -c ${card} cset name='PCM Playback Volume' ${level}%`);
        });
    }
    recordAudio(name) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`arecord -f cd ${name}.wav`);
        });
    }
    stopRecording() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`pkill arecord`);
        });
    }
    listCaptureDevices() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`arecord -l | grep -E '^[^  ][^**]'`)
                .then(success => {
                if (success.length === 0) {
                    throw new Error('No capture devices found. Check if your mic is correctly plugged.');
                }
                const devices = String(success).split('\n');
                if (devices.length === 0) {
                    throw new Error('No capture devices found. Check if your mic is correctly plugged.');
                }
                try {
                    return devices.map(device => new audiodevice_1.AudioDevice(device));
                }
                catch (e) {
                    throw new Error('Error reading the information of the audio device: ' + e.message);
                }
            });
        });
    }
    listOuputDevices() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`aplay -l | grep -E '^[^  ][^**]'`)
                .then(success => {
                const devices = String(success).split('\n');
                if (devices.length === 0) {
                    throw new Error('No output devices found. This should not happen, on your device, run \narecord -l');
                }
                try {
                    return devices.map(device => new audiodevice_1.AudioDevice(device));
                }
                catch (e) {
                    throw new Error('Error reading the information of the audio device: ' + e.message);
                }
            });
        });
    }
    launchSnipsWatch(listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog(`snips-watch -v`, listener);
        });
    }
    journalctlLogs(services, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (services !== undefined && services.length > 0) {
                const s = services.map(service => ` -u ${service}`);
                return this.sshCommandLog(`journalctl -f ${s}`, listener);
            }
            return this.sshCommandLog(`journalctl -f -u "snips*"`, listener);
        });
    }
    snipsServicesSystemctlStatus() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return Promise.all(exports.availableSnipsServices.map((serviceName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const active = yield this.systemctlStatus(serviceName);
                return { active, name: serviceName };
            })));
        });
    }
    systemctlStatus(service) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.ssh.exec(`systemctl is-active ${service}.service >/dev/null 2>&1 && echo 'active' || echo 'inactive'`)
                .then(result => result === 'active');
        });
    }
    playAudio(name) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`aplay ${name}.wav`);
        });
    }
    removeAudioFile(name) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`rm ${name}.wav`);
        });
    }
    setupAsoundConf(outputDevice, captureDevice, isRespeaker) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const output = isRespeaker === true ? 'dmix' : 'hw';
            return this.ssh.exec(`(echo 'pcm.!default {'; \
        echo '  type asym'; \
        echo '  playback.pcm {'; \
        echo '    type plug'; \
        echo '    slave.pcm "${output}:${outputDevice.card},${outputDevice.subdevice}"'; \
        echo '  }'; \
        echo '  capture.pcm {'; \
        echo '    type plug
        '; \
        echo '    slave.pcm "dsnoop:${captureDevice.card},${captureDevice.subdevice}"'; \
        echo '  }'; \
        echo '}') \
        | sudo tee /etc/asound.conf > /dev/null`);
        });
    }
    copySnipsfile(snipsfile) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`echo "${snipsfile}" | sudo tee ~/Snipsfile > /dev/null`);
        });
    }
    runSnipsmanagerInstall(username, password, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog(`sudo cp /etc/asound.conf /etc/asound.conf.back`, listener)
                .catch(_ => { })
                .then(_ => {
                if (username.length > 0 && password.length > 0) {
                    return this.sshCommandLog(`sudo snipsmanager install --skip-bluetooth --force-download --email=${username} --password=${password}`, listener);
                }
                return this.sshCommandLog(`sudo snipsmanager install --skip-bluetooth --force-download`, listener);
            })
                .then(_ => {
                return this.sshCommandLog(`sudo cp /etc/asound.conf.back /etc/asound.conf`, listener);
            });
        });
    }
    relaunchService(service) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo systemctl restart ${service}.service`);
        });
    }
    installPip(url, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog(`sudo /opt/venvs/snipsmanager/bin/pip install git+${url}`, listener);
        });
    }
    runSnipsManager(listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog(`sudo systemctl restart snipsmanager.service`, listener);
        });
    }
    updateSnips(listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog('sudo apt-get update; sudo apt-get -o Dpkg::Options::="--force-confold" upgrade -y snips-platform-voice', listener);
        });
    }
    installSnips(listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            listener('Updating aptitude repository');
            return this.sshCommandLog('sudo apt-get update', listener)
                .then(_ => {
                return this.sshCommandLog('sudo apt-get install -y dirmngr', listener)
                    .catch(_ => { throw new Error(`Couldn't install dirmngr`); });
            })
                .then(_ => {
                return this.sshCommandLog(`sudo bash -c 'echo "deb https://raspbian.snips.ai/$(lsb_release -cs) stable main" > /etc/apt/sources.list.d/snips.list'`, listener)
                    .catch(_ => { throw new Error(`Couldn't update Snips apt sources on the device`); });
            })
                .then(_ => {
                let addSnipsKeyCommand = '';
                try {
                    const snipsPGPPubKey = pgpKey_1.default.read();
                    addSnipsKeyCommand = `(echo "${snipsPGPPubKey}" > ~/snips-pgp-key.asc) && sudo apt-key add ~/snips-pgp-key.asc && rm snips-pgp-key.asc`;
                }
                catch (e) {
                    addSnipsKeyCommand = `sudo apt-key adv --keyserver pgp.mit.edu --recv-keys D4F50CDCA10A2849`;
                }
                return this.ssh.exec(addSnipsKeyCommand)
                    .catch(err => {
                    if (err.message.includes(`gpg: key D4F50CDCA10A2849: "Snips Raspbian distribution <infra@snips.ai>" not changed`) ||
                        err.message.includes(`Warning: apt-key output should not be parsed (stdout is not a terminal)`)) {
                        return;
                    }
                    throw new Error(`Couldn't fetch the PGP certificate to authenticate Snips APT repository.\nTry again and if the problem persists, please contact us`);
                });
            })
                .then(_ => {
                listener('Installing Snips Platform');
                return this.sshCommandLog(`sudo apt-get update; sudo apt-get install -y snips-platform-voice snips-watch snipsmanager python-dev`, listener)
                    .catch(err => { throw new Error(`There was a problem during Snips Platform installation ${err}`); });
            });
        });
    }
    packagesVersion() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const formatOutput = (dpkgOutput) => {
                return dpkgOutput
                    .split('\n')
                    .map(line => {
                    const split = line.trim().split(' ');
                    if (split.length >= 2)
                        return { name: split[0], version: split[split.length - 1] };
                    return { name: '', version: '' };
                })
                    .filter(v => v.name !== '');
            };
            return this.ssh.exec(`dpkg -l | grep snips | awk '{printf("%- 30s %- 10s\\n", $2,$3)}'`)
                .then(output => {
                return formatOutput(output).filter(v => exports.availableSnipsServices.includes(v.name));
            });
        });
    }
    updateWifi(name, passphrase) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`(echo ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev; \
            echo update_config=1; \
            echo ; \
            wpa_passphrase \"${name}\" \"${passphrase}\") \
            | sudo tee /etc/wpa_supplicant/wpa_supplicant.conf > /dev/null`)
                .then(_ => {
                return this.ssh.exec('sudo wpa_cli reconfigure');
            });
        });
    }
    statusWifi() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec('sudo wpa_cli status')
                .then(response => {
                let ssid;
                let status;
                response.split('\n')
                    .map(line => line.split('='))
                    .forEach(lineComponents => {
                    const key = lineComponents[0];
                    const value = lineComponents[1];
                    if (key === 'ssid') {
                        ssid = value;
                    }
                    else if (key === 'wpa_state') {
                        status = value;
                    }
                });
                return [ssid, status];
            });
        });
    }
    reconnectWifi() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec('sudo wpa_cli reconnect');
        });
    }
    disconnectWifi() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec('sudo wpa_cli disconnect');
        });
    }
    putFile(localPath, remotePath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.putFile(localPath, remotePath);
        });
    }
    putDirectory(localDirectory, remoteDirectory) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.ssh.putDirectory(localDirectory, remoteDirectory, {
                recursive: true,
                concurrency: 1,
                validate: itemPath => {
                    const baseName = path.basename(itemPath);
                    return baseName.substr(0, 1) !== '.';
                },
            });
        });
    }
    removeFile(remotePath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo rm ${remotePath}`);
        });
    }
    removeDirectory(remotePath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo rm -rf ${remotePath}`);
        });
    }
    unzipFile(filePath, destinationPath = '.') {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`unzip ${filePath} -d ${destinationPath}`);
        });
    }
    installAssistant(assistantPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const assistantFilename = path.basename(assistantPath);
            return this.removeDirectory('/usr/share/snips/assistant')
                .then(_ => {
                if (assistantFilename.includes('zip')) {
                    return this.putFile(assistantPath, `/tmp/${assistantFilename}`)
                        .then(_ => this.unzipFile(`/tmp/${assistantFilename}`, '/tmp/'))
                        .then(_ => this.removeFile(`/tmp/${assistantFilename}`));
                }
                return this.putDirectory(assistantPath, `/tmp/assistant`)
                    .then(status => {
                    if (!status) {
                        throw new Error('Some files failed to transfer, aborting');
                    }
                    return 'success';
                });
            })
                .then(_ => this.ssh.exec(`sudo mv /tmp/assistant /usr/share/snips/assistant`));
        });
    }
    dpkgCheckIfPackageIsInstalled(packageName) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`dpkg-query -W -f='${packageName}' nano 2>/dev/null | grep -c "ok installed"`)
                .then(result => result === 1 ? true : false);
        });
    }
    installRespeakerMicHat(listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog('git clone https://github.com/respeaker/seeed-voicecard.git', listener)
                .then(_ => this.sshCommandLog('cd seeed-voicecard; git checkout b9d9cde471358855ff66b3d18eb3d0f2c8a73987; sudo ./install.sh 2mic; sudo reboot', listener));
        });
    }
    sshCommandLog(command, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.execCommand(command, {
                onStdout(chunk) {
                    listener(chunk.toString('utf8'));
                },
                onStderr(chunk) {
                    listener(chunk.toString('utf8'));
                },
            });
        });
    }
    playWAVtoASR(remoteWAVPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`cat ${remoteWAVPath} | nc localhost 1234`);
        });
    }
    journalctlLog(service, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.sshCommandLog(`journalctl -S now -o cat -f -u ${service}.service`, listener);
        });
    }
    editAudioServerService() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-audio-server.*#ExecStart=/usr/bin/snips-audio-server --hijack localhost:1234 --nomike#g" /lib/systemd/system/snips-audio-server.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-audio-server.service`);
        });
    }
    resetAudioServerService() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-audio-server.*#ExecStart=/usr/bin/snips-audio-server#g" /lib/systemd/system/snips-audio-server.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-audio-server.service`);
        });
    }
    editASRService(maxActive, minActive) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-asr.*#ExecStart=/usr/bin/snips-asr --max_active ${maxActive} --min_active ${minActive}#g" /lib/systemd/system/snips-asr.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-asr.service`);
        });
    }
    resetASRService() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-asr.*#ExecStart=/usr/bin/snips-asr#g" /lib/systemd/system/snips-asr.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-asr.service`);
        });
    }
    publishMQTTStartListening() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec(`mosquitto_pub -h 127.0.0.1 -t 'hermes/asr/startListening' -m '{ "siteId":"default", "sessionId": "some_session" }'`);
        });
    }
    killNCServer() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.ssh.exec('pkill nc');
        });
    }
    sshCopyId() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let sshPubKeyFile;
            if (fsExtra.existsSync(snipsRSAKeyFilePath) === false) {
                sshPubKeyFile = yield this.sshKeygen();
                this.addSnipsRSAKeyToSSHConfigIfNeeded();
            }
            else {
                sshPubKeyFile = fsExtra.readFileSync(snipsRSAKeyFilePath + '.pub', 'utf8');
                this.addSnipsRSAKeyToSSHConfigIfNeeded();
            }
            const command = `echo "${sshPubKeyFile}" | exec sh -c 'cd ; umask 077 ; mkdir -p .ssh && cat >> ${exports.snipsRemoteAuthorizedKeysFilename} || exit 1 ; if type restorecon >/dev/null 2>&1 ; then restorecon -F .ssh ${exports.snipsRemoteAuthorizedKeysFilename} ; fi'`;
            return this.ssh.exec(command);
        });
    }
    ssh_keygen_binPath() {
        if (process.platform !== 'win32') {
            return 'ssh-keygen';
        }
        switch (process.arch) {
            case 'ia32': return path.join(__dirname, '..', 'bin', 'ssh-keygen-32.exe');
            case 'x64': return path.join(__dirname, '..', 'bin', 'ssh-keygen-64.exe');
        }
        throw new Error('Unsupported platform');
    }
    sshKeygen() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const command = `${this.ssh_keygen_binPath()} -t rsa -b 2048 -C "Snips RSA key" -N "" -f ${snipsRSAKeyFilePath}`;
            shelljs.exec(command, { async: false, silent: false });
            try {
                return fsExtra.readFileSync(snipsRSAKeyFilePath + '.pub', 'utf8');
            }
            catch (e) {
                throw new Error(`Snips RSA key couldn't be found`);
            }
        });
    }
    addSnipsRSAKeyToSSHConfigIfNeeded() {
        const sshConfigPath = os.homedir() + '/.ssh/config';
        const identityFilePath = '  IdentityFile ~/.ssh/' + snipsRSAKeyName;
        let sshConfigFile;
        try {
            const sshConfigFileLines = fsExtra.readFileSync(sshConfigPath, 'utf8').split('\n');
            if (sshConfigFileLines.indexOf(identityFilePath) >= 0) {
                return;
            }
            const hostStarIndex = sshConfigFileLines.indexOf('Host *');
            if (hostStarIndex >= 0) {
                sshConfigFileLines.splice(hostStarIndex + 1, 0, identityFilePath);
            }
            else {
                sshConfigFileLines.push('Host *');
                sshConfigFileLines.push(identityFilePath);
            }
            sshConfigFile = sshConfigFileLines.join('\n');
        }
        catch (e) {
            sshConfigFile = `Host *\n  AddKeysToAgent yes\n${identityFilePath}`;
        }
        fsExtra.writeFileSync(sshConfigPath, sshConfigFile);
    }
}
exports.SSHService = SSHService;
