import chalk from 'chalk';
import * as yargs from 'yargs';
import { cli } from '../../cli';
import { AudioDevice } from '../../utils/audiodevice';
import { SSHService } from '../../session/ssh';

exports.command = 'audio';
exports.desc = 'Quickly setup the mic & speaker on your device';

exports.handler = async (argv: yargs.Argv) => {
    const ssh = new SSHService();
    await ssh.connect()
    .catch(e => {
        cli.stream.error(e);
        process.exit();
    });

    const respeaker = await cli.prompt.promptQuestions([
        {
            name: 'respeaker',
            type: 'confirm',
            message: 'Do you have a Respeaker mic hat (2mic or 4mic) ?',
            default: false,
        },
    ]);

    if (respeaker['respeaker'] === true) {
        const respeakerbis = await cli.prompt.promptQuestions([
            {
                name: 'installation',
                type: 'confirm',
                message: `Do you want to install the Respeaker's drivers ?`,
                default: false,
            },
        ]);

        if (respeakerbis['installation'] === true) {
            await ssh.installRespeakerMicHat(output => cli.stream.print(output))
            .catch(e => cli.stream.error(e));

            let loadingText = 'Waiting for device to reboot';
            cli.stream.loading(loadingText);
            await ssh.retryConnect(() => {
                loadingText += ' . ';
                cli.stream.update(loadingText);
            })
            .then(isConnected => {
                if (isConnected) {
                    cli.stream.done();
                    cli.stream.success('Device has rebooted');
                } else {
                    cli.stream.error('Failed to reconnect, try a hard reboot');
                    process.exit();
                }
            });
        }
    }

    let captureDevice: AudioDevice | undefined;
    try {
        const captureDevices = await ssh.listCaptureDevices();
        if (captureDevices.length === 1) {
            cli.stream.success(`Found only 1 capture device named: ${captureDevices[0].name}`);
            cli.stream.println(`We will use this device to capture sound`);
            captureDevice = captureDevices[0];
        } else {
            const response = await cli.prompt.promptList(
                'device',
                `We found ${captureDevices.length} interfaces to record sound.\n  Choose the one you want to use`,
                captureDevices.map(device => device.name),
            );
            captureDevice = captureDevices.find(device => device.name === response['device']);
        }
    } catch (e) {
        cli.stream.error('Error fetching audio capture device: ' + e.message);
    }
    if (captureDevice === undefined) {
        cli.stream.hint('Using default capture configuration card 1 subdevice 0');
        captureDevice = { name: 'Default', card: 1, subdevice: 0 };
    }

    let outputDevice: AudioDevice | undefined;
    try {
        const outputDevices = await ssh.listOuputDevices();
        if (outputDevices.length === 1) {
            cli.stream.success(`Found only 1 output device named: ${outputDevices[0].name}`);
            cli.stream.println(`We will use this device to output sound`);
            outputDevice = outputDevices[0];
        } else {
            const response = await cli.prompt.promptList(
                'device',
                `We found ${outputDevices.length} interfaces to output sound.\n  Choose the one you want to use as your speaker`,
                outputDevices.map(device => device.name),
            );
            outputDevice = outputDevices.find(device => device.name === response['device']);
        }
    } catch (e) {
        cli.stream.error('Error fetching audio output device: ' + e.message);
    }
    if (outputDevice === undefined) {
        cli.stream.error(`An output device wasn't found`);
        return;
    }

    try {
        await ssh.setupAsoundConf(outputDevice, captureDevice, respeaker['respeaker']);
        cli.stream.success('Installed /etc/asound.conf');
    } catch (e) {
        cli.stream.error('Error installing the microphone asound.conf: ' + e.message);
        ssh.disconnect();
        process.exit();
    }

    cli.stream.loading('Restarting snips-audio-server service');
    await ssh.relaunchService('snips-audio-server').catch(_ => {});
    cli.stream.done();

    cli.stream.loading('Restarting snipsmanager service');
    await ssh.relaunchService('snipsmanager').catch(_ => {});
    cli.stream.done();

    cli.stream.hint(chalk`You can test your microphone & speaker with {blue sam test microphone / speaker}`);
    ssh.disconnect();
};
