import fsExtra = require('fs-extra');
import nodeSsh = require('node-ssh');
import os = require('os');
import path = require('path');
import shelljs = require('shelljs');
import { AudioDevice } from '../utils/audiodevice';
import { Assistant } from '../models/assistant';
import { Credentials, Config } from '../utils/config';
import { newError, NO_CREDENTIALS_FOUND, SSH_CONNECT_ERROR } from '../utils/errors';
import { timeout } from '../utils/promises';
import pgpKey from '../assets/pgpKey';

export const snipsRemoteAuthorizedKeysFilename = '~/.ssh/authorized_keys';

const snipsRSAKeyName = 'id_rsa_snips';
const snipsRSAKeyFilePath = path.join(os.homedir(), '.ssh', snipsRSAKeyName);

export const availableSnipsServices = ['snips-hotword', 'snips-queries', 'snips-analytics', 'snips-dialogue', 'snips-asr', 'snips-tts', 'snips-audio-server', 'snipsmanager'];

export class SSHService {

    private ssh: any;
    credentials: Credentials;

    constructor() {
        this.ssh = new nodeSsh();
    }

    public async connect(): Promise<string> {
        const credentials: Credentials | void = await Config.getCredentials()
        .catch(_ => { newError(`No credentials found`, NO_CREDENTIALS_FOUND); });

        if (credentials === undefined) {
            throw newError(`No credentials found`, NO_CREDENTIALS_FOUND);
        }

        return this.connectInternal(credentials)
        .catch(e => {
            this.disconnect();
            throw newError(e.message, SSH_CONNECT_ERROR);
        });
    }

    public async retryConnect(tick: () => void, limit = 5): Promise<boolean> {
        let isConnected = false;
        let numberOfRetries = 0;
        while (isConnected === false && limit > numberOfRetries) {
            await timeout(5000);
            tick();
            await this.connect()
            .then(_ =>  isConnected = true)
            .catch(_ => {});
            numberOfRetries += 1;
        }
        return isConnected;
    }

    public async newConnection(credentials: Credentials, password: string, onConnected: () => void, onSSHCopyId: () => void, onCredentialsSaved: () => void): Promise<void> {
        await this.connectInternal(credentials, password)
        .then(_ => onConnected())
        .catch(e => {
            throw new Error(`Error connecting to ${credentials.hostname}: ${e.message}`);
        });

        await this.sshCopyId()
        .then(_ => onSSHCopyId())
        .catch(e => {
            this.disconnect();
            throw new Error(`Failed to copy SSH public key: ${e.message}`);
        });

        await Config.deleteCredentials().catch(_ => { });
        await Config.saveCredentialsOnDisk(credentials)
        .then(_ => onCredentialsSaved())
        .catch(e => {
            this.disconnect();
            throw new Error(`Failed to save credentials: ${e.message}`);
        });
    }

    private async connectInternal (credentials: Credentials, password?: string): Promise<any> {
        this.credentials = credentials;

        if (password != null) {
            return this.ssh.connect({
                password,
                host: this.credentials.hostname,
                username: this.credentials.username,

            });
        }
        return this.ssh.connect({
            host: this.credentials.hostname,
            username: this.credentials.username,
            privateKey: snipsRSAKeyFilePath,
        });
    }

    public async disconnect(): Promise<void> {
        await this.ssh.dispose();
    }

    public async reboot(): Promise<void> {
        this.ssh.exec('sudo reboot');
    }

    public async isAssistantInstalled(): Promise<boolean> {
        return this.ssh.exec(`if [ -d /usr/share/snips/assistant ]; then echo 1; else echo 0; fi`)
        .then(result => { return result === 1; });
    }

    public async getInstalledAssistant(): Promise<Assistant | undefined> {
        const assistantFilename = '/usr/share/snips/assistant/assistant.json';
        return this.ssh.exec(`if [ -f ${assistantFilename} ]; then cat ${assistantFilename}; else echo ""; fi`)
        .then(result => {
            if (result === '') { return undefined; }
            return Assistant.fromString(result);
        });
    }

    public async getOSVersion(): Promise<string> {
        return this.ssh.exec('cat /etc/os-release')
        .then(success => {
            const osVersion = success.split('\n')[0].replace('PRETTY_NAME="', '').replace('"', '');
            if (osVersion === null || osVersion.length === 0) {
                return 'Unknown';
            }
            return osVersion;
        })
        .catch(_ => {
            return 'Unknown';
        });
    }

    public async installSnipsPlatformDemo(listener: (output: string) => void): Promise<string> {
        return this.sshCommandLog(`sudo apt-get install -y snips-platform-demo`, listener);
    }

    public async aptGet(packageName: string, listener: (output: string) => void): Promise<string> {
        return this.sshCommandLog(`sudo apt-get install -y ${packageName}`, listener);
    }

    public async isNetcatInstalled(): Promise<boolean> {
        return this.ssh.exec(`if [ -x "$(command -v nc)" ]; then echo 1; else echo 0; fi`)
        .then(result => {
            if (!result) {
                return this.dpkgCheckIfPackageIsInstalled('netcat');
            }
            return 1;
        })
        .then(result => result === 1 ? true : false);
    }

    public async speakerTest(): Promise<string> {
        return this.ssh.exec(`sudo speaker-test -c2 -l2 -twav`);
    }

    public async stopSpeakerTest(): Promise<string> {
        return this.ssh.exec(`pkill speaker-test`);
    }

    public async setHostname(newHostname: string): Promise<string> {
        const oldHostname = await this.ssh.exec('cat /etc/hostname')
        .catch(_ => {
            throw new Error('You device hostname was not readable');
        });

        return this.ssh.exec(`sudo sed -i "s/${oldHostname}/${newHostname}/g" /etc/hosts; sudo sed -i "s/${oldHostname}/${newHostname}/g" /etc/hostname; sudo reboot`)
        .then(async _ => {
            this.credentials.hostname = newHostname + '.local';
            return Config.saveCredentialsOnDisk(this.credentials);
        })
        .catch(async error => {
            if (error.message.includes('sudo: unable to resolve host')) {
                this.credentials.hostname = newHostname + '.local';
                return Config.saveCredentialsOnDisk(this.credentials);
            }
        });
    }

    public static async fetchBootHostname(ips: string[]): Promise<{ ip: string, hostname: string, snipsHostname: string }[]> {
        return await Promise.all(
            ips.map(async ip => {
                const client = new nodeSsh();
                const hostnames = await client.connect({
                    host: ip,
                    username: 'pi',
                    password: 'raspberry',
                })
                .then(async _ => {
                    let snipsHostname: string = await client.exec('cat /boot/alias').catch(_ => '');
                    const hostname: string = await client.exec('cat /etc/hostname').catch(_ => '');
                    if (snipsHostname.includes('No such file or directory')) {
                        snipsHostname = '';
                    }
                    return { hostname, snipsHostname };
                })
                .catch(_ => {
                    return { snipsHostname: '', hostname: '' };
                });
                return { ip, hostname: hostnames.hostname, snipsHostname: hostnames.snipsHostname };
            }),
        );
    }

    public async getCaptureLevel(card: number | undefined = 1): Promise<string> {
        return this.ssh.exec(`amixer -c ${card} cget name='Mic Capture Volume'`);
    }

    public async setCaptureLevel(card: number | undefined = 1, level: number | undefined = 80): Promise<string> {
        return this.ssh.exec(`amixer -c ${card} cset name='Mic Capture Volume' ${level}%`);
    }

    public async getVolumeLevel(card: number | undefined = 0): Promise<string> {
        return this.ssh.exec(`amixer -c ${card} cget name='PCM Playback Volume'`);
    }

    public async setVolumeLevel(card: number | undefined = 0, level: number | undefined = 90): Promise<string> {
        return this.ssh.exec(`amixer -c ${card} cset name='PCM Playback Volume' ${level}%`);
    }

    public async recordAudio(name: string): Promise<string> {
        return this.ssh.exec(`arecord -f cd ${name}.wav`);
    }

    public async stopRecording(): Promise<string> {
        return this.ssh.exec(`pkill arecord`);
    }

    public async listCaptureDevices(): Promise<AudioDevice[]> {
        return this.ssh.exec(`arecord -l | grep -E '^[^  ][^**]'`)
        .then(success => {
            // card 0: ALSA [bcm2835 ALSA], device 0: bcm2835 ALSA [bcm2835 ALSA]
            if (success.length === 0) {
                throw new Error('No capture devices found. Check if your mic is correctly plugged.');
            }
            const devices = String(success).split('\n');
            if (devices.length === 0) {
                throw new Error('No capture devices found. Check if your mic is correctly plugged.');
            }
            try {
                return devices.map(device => new AudioDevice(device));
            } catch (e) {
                throw new Error('Error reading the information of the audio device: ' + e.message);
            }
        });
    }

    public async listOuputDevices(): Promise<AudioDevice[]> {
        return this.ssh.exec(`aplay -l | grep -E '^[^  ][^**]'`)
        .then(success => {
            const devices = String(success).split('\n');
            if (devices.length === 0) {
                throw new Error('No output devices found. This should not happen, on your device, run \narecord -l');
            }
            try {
                return devices.map(device => new AudioDevice(device));
            } catch (e) {
                throw new Error('Error reading the information of the audio device: ' + e.message);
            }
        });
    }

    public async launchSnipsWatch(listener: (string) => void): Promise<string> {
        return this.sshCommandLog(`snips-watch -v`, listener);
    }

    public async journalctlLogs(services: string[] | undefined, listener: (string) => void): Promise<string> {
        if (services !== undefined && services.length > 0) {
            const s = services.map(service => ` -u ${service}`);
            return this.sshCommandLog(`journalctl -f ${s}`, listener);
        }
        return this.sshCommandLog(`journalctl -f -u "snips*"`, listener);
    }

    public async snipsServicesSystemctlStatus(): Promise<{ name: string, active: boolean }[]> {
        return Promise.all(availableSnipsServices.map(async serviceName => {
            const active = await this.systemctlStatus(serviceName);
            return { active, name: serviceName };
        }));
    }

    public async systemctlStatus(service: string): Promise<boolean> {
        return await this.ssh.exec(`systemctl is-active ${service}.service >/dev/null 2>&1 && echo 'active' || echo 'inactive'`)
            .then(result => result === 'active');
    }

    public async playAudio(name: string): Promise<string> {
        // Playing audio throws back an error even though it executes the file. We ignore the error.
        return this.ssh.exec(`aplay ${name}.wav`);
        // .catch((e) => { });
    }

    public async removeAudioFile(name: string): Promise<string> {
        return this.ssh.exec(`rm ${name}.wav`);
    }

    public async setupAsoundConf(outputDevice: AudioDevice, captureDevice: AudioDevice, isRespeaker: boolean): Promise<string> {
        const output = isRespeaker === true ? 'dmix' : 'hw';
        return this.ssh.exec(`(echo 'pcm.!default {'; \
        echo '  type asym'; \
        echo '  playback.pcm {'; \
        echo '    type plug'; \
        echo '    slave.pcm "${output}:${outputDevice.card},${outputDevice.subdevice}"'; \
        echo '  }'; \
        echo '  capture.pcm {'; \
        echo '    type plug
        '; \
        echo '    slave.pcm "dsnoop:${captureDevice.card},${captureDevice.subdevice}"'; \
        echo '  }'; \
        echo '}') \
        | sudo tee /etc/asound.conf > /dev/null`);
    }

    public async copySnipsfile(snipsfile: string) {
        return this.ssh.exec(`echo "${snipsfile}" | sudo tee ~/Snipsfile > /dev/null`);
    }

    public async runSnipsmanagerInstall(username: string, password: string, listener: (string) => void) {
        return this.sshCommandLog(`sudo cp /etc/asound.conf /etc/asound.conf.back`, listener)
        .catch(_ => {})
        .then(_ => {
            if (username.length > 0 && password.length > 0) {
                return this.sshCommandLog(`sudo snipsmanager install --skip-bluetooth --force-download --email=${username} --password=${password}`, listener);
            }
            return this.sshCommandLog(`sudo snipsmanager install --skip-bluetooth --force-download`, listener);
        })
        .then(_ => {
            return this.sshCommandLog(`sudo cp /etc/asound.conf.back /etc/asound.conf`, listener);
        });
    }

    public async relaunchService(service: string): Promise<string> {
        return this.ssh.exec(`sudo systemctl restart ${service}.service`);
    }

    // TODO: temp hack since remote "snipsmanager install" fails on installing pip packages
    public async installPip(url: string, listener: (string) => void) {
        return this.sshCommandLog(`sudo /opt/venvs/snipsmanager/bin/pip install git+${url}`, listener);
    }

    public async runSnipsManager(listener: (string) => void) {
        return this.sshCommandLog(`sudo systemctl restart snipsmanager.service`, listener);
    }

    public async updateSnips(listener: (string) => void): Promise<string> {
        return this.sshCommandLog('sudo apt-get update; sudo apt-get -o Dpkg::Options::="--force-confold" upgrade -y snips-platform-voice', listener);
    }

    public async installSnips(listener: (string) => void): Promise<string> {
        listener('Updating aptitude repository');
        return this.sshCommandLog('sudo apt-get update', listener)
        .then(_ => {
            return this.sshCommandLog('sudo apt-get install -y dirmngr', listener)
            .catch(_ => { throw new Error(`Couldn't install dirmngr`); });
        })
        .then(_ => {
            return this.sshCommandLog(`sudo bash -c 'echo "deb https://raspbian.snips.ai/$(lsb_release -cs) stable main" > /etc/apt/sources.list.d/snips.list'`, listener)
            .catch(_ => { throw new Error(`Couldn't update Snips apt sources on the device`); });
        })
        .then(_ => {
            let addSnipsKeyCommand = '';
            try {
                const snipsPGPPubKey = pgpKey.read();
                addSnipsKeyCommand = `(echo "${snipsPGPPubKey}" > ~/snips-pgp-key.asc) && sudo apt-key add ~/snips-pgp-key.asc && rm snips-pgp-key.asc`;
            } catch (e) {
                addSnipsKeyCommand = `sudo apt-key adv --keyserver pgp.mit.edu --recv-keys D4F50CDCA10A2849`;
            }
            return this.ssh.exec(addSnipsKeyCommand)
            .catch(err => {
                if (err.message.includes(`gpg: key D4F50CDCA10A2849: "Snips Raspbian distribution <infra@snips.ai>" not changed`) ||
                err.message.includes(`Warning: apt-key output should not be parsed (stdout is not a terminal)`)) {
                    return;
                }
                throw new Error(`Couldn't fetch the PGP certificate to authenticate Snips APT repository.\nTry again and if the problem persists, please contact us`);
            });
        })
        .then(_ => {
            listener('Installing Snips Platform');
            // TODO: Remove dependency on python-dev (this is a hack we did to support snips-skill-owm
            // that depends on Python-Levenshtein that depends on python-dev. This dependency should be
            // put in the Debian dependencies of snipsmanager)
            return this.sshCommandLog(`sudo apt-get update; sudo apt-get install -y snips-platform-voice snips-watch snipsmanager python-dev`, listener)
            .catch(err => { throw new Error(`There was a problem during Snips Platform installation ${err}`); });
        });
    }

    public async packagesVersion(): Promise<{ name: string, version: string }[]> {
        const formatOutput = (dpkgOutput: string): { name: string, version: string }[] => {
            return dpkgOutput
            .split('\n')
            .map(line => {
                const split = line.trim().split(' ');
                if (split.length >= 2) return { name: split[0], version: split[split.length - 1] };
                return { name: '', version: '' };
            })
            .filter(v => v.name !== '');
        };
        return this.ssh.exec(`dpkg -l | grep snips | awk '{printf("%- 30s %- 10s\\n", $2,$3)}'`)
        .then(output => {
            return formatOutput(output).filter(v => availableSnipsServices.includes(v.name));
        });
    }

    public async updateWifi(name: string, passphrase: string): Promise<string> {
        return this.ssh.exec(`(echo ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev; \
            echo update_config=1; \
            echo ; \
            wpa_passphrase \"${name}\" \"${passphrase}\") \
            | sudo tee /etc/wpa_supplicant/wpa_supplicant.conf > /dev/null`)
            .then(_ => {
                return this.ssh.exec('sudo wpa_cli reconfigure');
            });
    }

    public async statusWifi(): Promise<string> {
        return this.ssh.exec('sudo wpa_cli status')
        .then(response => {
            let ssid: string | undefined;
            let status: string | undefined;
            response.split('\n')
            .map(line => line.split('='))
            .forEach(lineComponents => {
                const key = lineComponents[0];
                const value = lineComponents[1];
                if (key === 'ssid') {
                    ssid = value;
                } else if (key === 'wpa_state') {
                    // TODO add wpa_state
                    status = value;
                }
            });
            return [ssid, status];
        });
    }

    public async reconnectWifi(): Promise<string> {
        return this.ssh.exec('sudo wpa_cli reconnect');
    }

    public async disconnectWifi(): Promise<string> {
        return this.ssh.exec('sudo wpa_cli disconnect');
    }

    public async putFile(localPath: string, remotePath: string): Promise<string> {
        return this.ssh.putFile(localPath, remotePath);
    }

    public async putDirectory(localDirectory: string, remoteDirectory: string): Promise<string> {
        return await this.ssh.putDirectory(localDirectory, remoteDirectory, {
            recursive: true,
            concurrency: 1, // /!\ Anything >1 will result in potentially failed transfered files, you've been warned
            validate: itemPath => {
                const baseName = path.basename(itemPath);
                return baseName.substr(0, 1) !== '.'; // do not allow dot files
            },
        });
    }

    public async removeFile(remotePath: string): Promise<string> {
        return this.ssh.exec(`sudo rm ${remotePath}`);
    }

    public async removeDirectory(remotePath: string): Promise<string> {
        return this.ssh.exec(`sudo rm -rf ${remotePath}`);
    }

    public async unzipFile(filePath: string, destinationPath: string = '.'): Promise<string> {
        return this.ssh.exec(`unzip ${filePath} -d ${destinationPath}`);
    }

    public async installAssistant(assistantPath: string): Promise<string> {
        const assistantFilename: string = path.basename(assistantPath);

        return this.removeDirectory('/usr/share/snips/assistant')
        .then(_ => {
            if (assistantFilename.includes('zip')) {
                return this.putFile(assistantPath, `/tmp/${assistantFilename}`)
                .then(_ => this.unzipFile(`/tmp/${assistantFilename}`, '/tmp/'))
                .then(_ => this.removeFile(`/tmp/${assistantFilename}`));
            }
            return this.putDirectory(assistantPath, `/tmp/assistant`)
            .then(status => {
                // PutDirectory is not sending an explicit error, so we catch it here.
                if (!status) {
                    throw new Error('Some files failed to transfer, aborting');
                }
                return 'success';
            });
        })
        .then(_ => this.ssh.exec(`sudo mv /tmp/assistant /usr/share/snips/assistant`));
    }

    public async dpkgCheckIfPackageIsInstalled(packageName: string): Promise<boolean> {
        return this.ssh.exec(`dpkg-query -W -f='${packageName}' nano 2>/dev/null | grep -c "ok installed"`)
        .then(result => result === 1 ? true : false);
    }

    public async installRespeakerMicHat(listener: (string) => void): Promise<string> {
        return this.sshCommandLog('git clone https://github.com/respeaker/seeed-voicecard.git', listener)
        .then(_ => this.sshCommandLog('cd seeed-voicecard; git checkout b9d9cde471358855ff66b3d18eb3d0f2c8a73987; sudo ./install.sh 2mic; sudo reboot', listener));
    }

    private async sshCommandLog(command: string, listener: (string) => void): Promise<string> {
        return this.ssh.execCommand(command, {
            onStdout(chunk) {
                listener(chunk.toString('utf8'));
            },
            onStderr(chunk) {
                listener(chunk.toString('utf8'));
            },
        });
    }

    /*----------------------------------------------------------------------*/
    // ASR Benchmark

    public async playWAVtoASR(remoteWAVPath: string): Promise<string> {
        return this.ssh.exec(`cat ${remoteWAVPath} | nc localhost 1234`);
    }

    public async journalctlLog(service: string, listener: (string) => void): Promise<string> {
        return this.sshCommandLog(`journalctl -S now -o cat -f -u ${service}.service`, listener);
    }

    public async editAudioServerService(): Promise<string> {
        return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-audio-server.*#ExecStart=/usr/bin/snips-audio-server --hijack localhost:1234 --nomike#g" /lib/systemd/system/snips-audio-server.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-audio-server.service`);
    }

    public async resetAudioServerService(): Promise<string> {
        return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-audio-server.*#ExecStart=/usr/bin/snips-audio-server#g" /lib/systemd/system/snips-audio-server.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-audio-server.service`);
    }

    public async editASRService(maxActive: number, minActive: number): Promise<string> {
        return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-asr.*#ExecStart=/usr/bin/snips-asr --max_active ${maxActive} --min_active ${minActive}#g" /lib/systemd/system/snips-asr.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-asr.service`);
    }

    public async resetASRService(): Promise<string> {
        return this.ssh.exec(`sudo sed -i "s#.*ExecStart=/usr/bin/snips-asr.*#ExecStart=/usr/bin/snips-asr#g" /lib/systemd/system/snips-asr.service; \
        sudo systemctl daemon-reload; \
        sudo systemctl restart snips-asr.service`);
    }

    public async publishMQTTStartListening(): Promise<string> {
        return this.ssh.exec(`mosquitto_pub -h 127.0.0.1 -t 'hermes/asr/startListening' -m '{ "siteId":"default", "sessionId": "some_session" }'`);
    }

    public async killNCServer(): Promise<string> {
        return this.ssh.exec('pkill nc');
    }

    /*----------------------------------------------------------------------*/
    // RSA Key Generation

    public async sshCopyId(): Promise<string> {
        let sshPubKeyFile: string;
        if (fsExtra.existsSync(snipsRSAKeyFilePath) === false) {
            sshPubKeyFile = await this.sshKeygen();
            this.addSnipsRSAKeyToSSHConfigIfNeeded();
        } else {
            sshPubKeyFile = fsExtra.readFileSync(snipsRSAKeyFilePath + '.pub', 'utf8');
            this.addSnipsRSAKeyToSSHConfigIfNeeded();
        }

        // The ssh-copy-id script doesn't exist on Windows, the Unix script essentially copies the keys on the remote server with ssh
        // You can check the original code on your own system : cat /usr/bin/ssh-copy-id
        // The command below is the same as the osx version.
        const command = `echo "${sshPubKeyFile}" | exec sh -c 'cd ; umask 077 ; mkdir -p .ssh && cat >> ${snipsRemoteAuthorizedKeysFilename} || exit 1 ; if type restorecon >/dev/null 2>&1 ; then restorecon -F .ssh ${snipsRemoteAuthorizedKeysFilename} ; fi'`;
        return this.ssh.exec(command);
    }

    private ssh_keygen_binPath(): string {
        if (process.platform !== 'win32') { return 'ssh-keygen'; }

        switch (process.arch) {
            case 'ia32': return path.join(__dirname, '..', 'bin', 'ssh-keygen-32.exe');
            case 'x64': return path.join(__dirname, '..', 'bin', 'ssh-keygen-64.exe');
        }

        throw new Error('Unsupported platform');
    }

    private async sshKeygen(): Promise<string> {
        const command = `${this.ssh_keygen_binPath()} -t rsa -b 2048 -C "Snips RSA key" -N "" -f ${snipsRSAKeyFilePath}`;
        shelljs.exec(command, { async: false, silent: false });
        try {
            return fsExtra.readFileSync(snipsRSAKeyFilePath + '.pub', 'utf8');
        } catch (e) {
            throw new Error(`Snips RSA key couldn't be found`);
        }
    }

    private addSnipsRSAKeyToSSHConfigIfNeeded() {
        const sshConfigPath = os.homedir() + '/.ssh/config';
        const identityFilePath = '  IdentityFile ~/.ssh/' + snipsRSAKeyName;

        let sshConfigFile: string | undefined;
        try {
            const sshConfigFileLines = fsExtra.readFileSync(sshConfigPath, 'utf8').split('\n');
            if (sshConfigFileLines.indexOf(identityFilePath) >= 0) {
                return;
            }
            const hostStarIndex = sshConfigFileLines.indexOf('Host *');
            if (hostStarIndex >= 0) {
                sshConfigFileLines.splice(hostStarIndex + 1, 0, identityFilePath);
            } else {
                sshConfigFileLines.push('Host *');
                sshConfigFileLines.push(identityFilePath);
            }
            sshConfigFile = sshConfigFileLines.join('\n');
        } catch (e) {
            sshConfigFile = `Host *\n  AddKeysToAgent yes\n${identityFilePath}`;
        }
        fsExtra.writeFileSync(sshConfigPath, sshConfigFile);
    }
}
